package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;

/**
 * Array update operation.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class ArrayUpdate extends AbstractTerm {
 /** Array value. */
  private final Term array;
  
  /** Index value. */
  private final Term index;
  
  /** New element value. */
  private final Term value;
  
  /**
   * Constructor.
   * @param array Array value.
   * @param index Index value.
   * @param value New element value.
   */
  public ArrayUpdate(final Term array, 
                     final Term index, 
                     final Term value) {
    this.array = array;
    this.index = index;
    this.value = value;
  }

  /**
   * Get the array value.
   * @return Array.
   */
  public Term getArray() {
    return array;
  }

  /**
   * Get the index value.
   * @return Index.
   */
  public Term getIndex() {
    return index;
  }

  /**
   * Get the assigned value.
   * @return Value.
   */
  public Term getValue() {
    return value;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseArrayUpdate(array, index, value);
  }
}
