package net.afyre.haha.demo.tests;

import net.afyre.haha.demo.haha.HahaPackage;
import net.afyre.haha.demo.testsutils.HahaTestsInjectorProvider;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.xtext.junit4.InjectWith;
import org.eclipse.xtext.util.Pair;
import org.eclipselabs.xtext.utils.unittesting.FluentIssueCollection;
import org.eclipselabs.xtext.utils.unittesting.XtextRunner2;
import org.eclipselabs.xtext.utils.unittesting.XtextTest;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

@InjectWith(HahaTestsInjectorProvider.class)
@RunWith(XtextRunner2.class)
public class HahaModelTests extends XtextTest {
  public HahaModelTests() {
    super("platform:/plugin/net.afyre.haha.demo.tests/data");
  }
  
  @Before
  public void before() {
    //super.before();
    suppressSerialization();
    EPackage.Registry.INSTANCE.put(HahaPackage.eNS_URI,
                                   HahaPackage.eINSTANCE);
//    EcorePlugin.getEPackageNsURIToGenModelLocationMap().put(BusinessDomainDslPackage.eNS_URI,
//                                                            URI.createURI("classpath:/mod4j/BusinessDomainDsl.genmodel"));
  }

  @Test
  public void arrayAccess(){
    testParserRule("x[1]", "ArrayAccess");
    testParserRule("x[s][3]", "ArrayAccess");
    testParserRuleErrors("x]1]", "ArrayAccess", "missing EOF at ']'");
    testParserRuleErrors("x[1", "ArrayAccess", "mismatched input '<EOF>' expecting ']'");
    testParserRuleErrors("]", "ArrayAccess", "no viable alternative at input ']'");
    testParserRuleErrors("[", "ArrayAccess", "no viable alternative at input '['");
    testParserRuleErrors("[]", "ArrayAccess", "no viable alternative at input '['");
  }

  @Test
  public void boolLiteral(){
    testParserRule("⊤", "BoolLiteral");
    testParserRule("true", "BoolLiteral");
    testParserRule("false", "BoolLiteral");
    testParserRule("⊥", "BoolLiteral");
    
    testParserRuleErrors("truer", "BoolLiteral");
    testParserRuleErrors("falsef", "BoolLiteral");
  }
  
  @Test
  public void test_01_funpass() {
      testFile("testfiles/01-funpass.haha");
  }

  @Test
  public void test_02_funblock() {
      testFile("testfiles/02-funblock.haha");
  }  

  
  @Test
  public void test_03_predicatetrue() {
      testFile("testfiles/03-predicatetrue.haha");
  }  

  @Test
  public void test_04_funcargs() {
      testFile("testfiles/04-funcargs.haha");
  }  

  @Test
  public void test_05_funcdocs() {
      testFile("testfiles/05-funcdocs.haha");
  }  

  @Test
  public void test_06_funcpreconditions() {
      testFile("testfiles/06-funcpreconditions.haha");
  }

  @Test
  public void test_06_funcpreconditions_a() {
	  Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/06-funcpreconditions-a.haha");
	  Assert.assertTrue(result.getSecond().getSummary().indexOf("Couldn't resolve reference to Variable 'f'")>0);
  }
  
  @Test
  public void test_07_funcpostconditions() {
      testFile("testfiles/07-funcpostconditions.haha");
  }

  @Test
  public void test_08_funclocals() {
      testFile("testfiles/08-funclocals.haha");
  }

  @Test
  public void test_09_funcmorelocals() {
      testFile("testfiles/09-funcmorelocals.haha");
  }
  
  @Test
  public void test_10_funcassertion() {
      testFile("testfiles/10-funcassertion.haha");
  }
  
  @Test
  public void test_11_funcassignment() {
      testFile("testfiles/11-funcassignment.haha");
  }

  @Test
  public void test_12_funccond() {
      testFile("testfiles/12-funccond.haha");
  }

  @Test
  public void test_13_funcloop() {
      testFile("testfiles/13-funcloop.haha");
  }

  @Test
  public void test_13_funcloop_a() {
	  Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/13-funcloop-a.haha");
	  Assert.assertTrue(result.getSecond().getSummary().indexOf("BOOLEAN expected, integer found")>0);
  }
  
  @Test
  public void test_14_funcarrays() {
      testFile("testfiles/14-funcarrays.haha");
  }
  
  @Test
  public void test_15_funcarrassign() {
      testFile("testfiles/15-funcarrassign.haha");
  }
  
  @Test
  public void test_16_funccondelse() {
      testFile("testfiles/16-funccondelse.haha");
  }
  
  @Test
  public void test_17_funcloopinv() {
      testFile("testfiles/17-funcloopinv.haha");
  }
  
  @Test
  public void test_18_exprequiv1() {
      testFile("testfiles/18-exprequiv1.haha");
  }

  @Test
  public void test_18_exprequiv2() {
      testFile("testfiles/18-exprequiv2.haha");
  }
  
  @Test
  public void test_19_exprimpl1() {
    testFile("testfiles/19-exprimpl1.haha");
  }

  @Test
  public void test_19_exprimpl2() {
    testFile("testfiles/19-exprimpl2.haha");
  }
  
  @Test
  public void test_20_exprdisj1() {
    testFile("testfiles/20-exprdisj1.haha");
  }

  @Test
  public void test_20_exprdisj2() {
    testFile("testfiles/20-exprdisj2.haha");
  }
  
  @Test
  public void test_21_exprconj1() {
    testFile("testfiles/21-exprconj1.haha");
  }
  
  @Test
  public void test_21_exprconj2() {
    testFile("testfiles/21-exprconj2.haha");
  }
  
  @Test
  public void test_22_exprneg() {
    testFile("testfiles/22-exprneg.haha");
  }
  
  @Test
  public void test_23_exprquant() {
    testFile("testfiles/23-exprquant.haha");
  }
  
  @Test
  public void test_24_exprcompare() {
    testFile("testfiles/24-exprcompare.haha");
  }
  
  @Test
  public void test_25_exprsum() {
    testFile("testfiles/25-exprsum.haha");
  }
  
  @Test
  public void test_26_exprproduct() {
    testFile("testfiles/26-exprproduct.haha");
  }
  
  @Test
  public void test_27_expruminus() {
    testFile("testfiles/27-expruminus.haha");
  }
  
  @Test
  public void test_28_comments() {
    testFile("testfiles/28-comments.haha");
  }
  
  @Test
  public void test_29_scoperesult() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/29-scoperesult.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Couldn't resolve reference to Assignable 'x'")>0);
  }
  
  @Test
  public void test_30_varinprecondition() {
    testFile("testfiles/30-varinprecondition.haha");
  }
  
  @Test
  public void test_31_resinprecondition() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/31-resinprecondition.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Couldn't resolve reference to Variable 'f'")>0);
  }
  
  @Test
  public void test_32_wrongtypeforall() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/32-wrongtypeforall.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Quantification over this type is not allowed")>0);
  }

  @Test
  public void test_33_wrongtypeexists() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/33-wrongtypeexists.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Quantification over this type is not allowed")>0);
  }
  
  @Test
  public void test_34_quantifiersexpr1() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/34-quantifiersexpr1.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Quantifiers are not allowed in expressions.")>0);
  }

  @Test
  public void test_34_quantifiersexpr2() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/34-quantifiersexpr2.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Quantifiers are not allowed in expressions.")>0);
  }
  
  @Test
  public void test_35_predexpr() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/35-predexpr.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Predicate calls are not allowed in expressions.")>0);
  }
  
  @Test
  public void test_36_nonnumber() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/36-nonnumber.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Z expected, boolean found")>0);
  }
  
  @Test
  public void test_37_nonbool() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/37-nonbool.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("BOOLEAN expected, integer found")>0);
  }
  
  @Test
  public void test_38_arrasz() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/38-arrasz.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Variable x has type ARRAY OF Z, Z expected")>0);
  }
  
  @Test
  public void test_39_arrass() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/39-arrass.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("x : Z is not an array")>0);
  }

  @Test
  public void test_39_arrass_a() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/39-arrass-a.haha");
	System.out.println("39a:"+result.getSecond().getSummary());
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Z is not an array")>0);
  }
  
  @Test
  public void test_40_boolvsint1() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/40-boolvsint1.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("BOOLEAN expected, integer found")>0);
  }

  @Test
  public void test_40_boolvsint2() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/40-boolvsint2.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Z expected, boolean found")>0);
  }

  @Test
  public void test_40_boolvsint3() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/40-boolvsint3.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("BOOLEAN expected, number found")>0);
  }

  @Test
  public void test_40_boolvsint4() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/40-boolvsint4.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Z expected, number found")>0);
  }

  
  @Test
  public void test_40_boolvsint5() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/40-boolvsint5.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("BOOLEAN expected, number found")>0);
  }

  @Test
  public void test_40_boolvsint6() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/40-boolvsint6.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Z expected, boolean found")>0);
  }

  @Test
  public void test_41_simplevsarray1() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/41-simplevsarray.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Variable i has type BOOLEAN, Z expected")>0);
  }
  
  @Test
  public void test_42_predinexpr() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/42-predinexpr.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Predicate calls are not allowed in expressions.")>0);
  }

  @Test
  public void test_43_funcall() {
      testFile("testfiles/43-funcall.haha");
  }

  @Test
  public void test_43_funcall1() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/43-funcall1.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Function f returns Z, not BOOLEAN")>0);
  }

  @Test
  public void test_43_funcall2() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/43-funcall2.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("is neither a function nor a predicate")>0);
  }

  @Test
  public void test_43_funcall3() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/43-funcall3.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Function f expects 0 arguments, not 1")>0);
  }

  @Test
  public void test_43_funcall4() {
      testFile("testfiles/43-funcall4.haha");
  }

  @Test
  public void test_43_funcall5() {
    Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/43-funcall5.haha");
    Assert.assertTrue(result.getSecond().getSummary().indexOf("Function f expects 2 arguments, not 1")>0);
  }
  
  @Test
  public void test_44_arith1() {
    testFile("testfiles/44-arith1.haha");
  }

  @Test
  public void test_44_arith2() {
    testFile("testfiles/44-arith2.haha");
  }

  @Test
  public void test_44_arith3() {
    testFile("testfiles/44-arith3.haha");
  }

  @Test
  public void test_44_arith4() {
    testFile("testfiles/44-arith4.haha");
  }

  @Test
  public void test_45_compare1() {
    testFile("testfiles/45-compare1.haha");
  }

  @Test
  public void test_45_compare2() {
    testFile("testfiles/45-compare2.haha");
  }
  
  @Test
  public void test_45_compare3() {
    testFile("testfiles/45-compare3.haha");
  }
  
  @Test
  public void test_45_compare4() {
    testFile("testfiles/45-compare4.haha");
  }
  
  @Test
  public void test_45_compare5() {
    testFile("testfiles/45-compare5.haha");
  }

  @Test
  public void test_46_scopepred() {
    testFile("testfiles/46-scopepred.haha");
  }
  
  @Test
  public void test_47_scopeexists() {
    testFile("testfiles/47-scopeexists.haha");
  }

  @Test
  public void test_48_scopeforall() {
    testFile("testfiles/48-scopeforall.haha");
  }

  @Test
  public void test_49_scopecomposite() {
    testFile("testfiles/49-scopecomposite.haha");
  }

  @Test
  public void test_50_types() {
	Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/50-types.haha");
	System.out.println("50:"+result.getSecond().getSummary());
	Assert.assertTrue(result.getSecond().getSummary().indexOf("Variable y has type ARRAY OF Z, Z expected")>0);
  }

  @Test
  public void test_51_axiom() {
    testFile("testfiles/51-axiom.haha");
  }

  @Test
  public void test_52_counter() {
    testFile("testfiles/52-counter.haha");
  }

  @Test
  public void test_52_counter_a() {
	Pair<String,FluentIssueCollection> result = loadAndSaveModule(resourceRoot, "testfiles/52-counter-a.haha");
	System.out.println("52a:"+result.getSecond().getSummary());
	Assert.assertTrue(result.getSecond().getSummary().indexOf("Counter variables are not allowed in expressions.")>0);
  }
  
  @Test
  public void test_ex_division() {
    testFile("examples/division.haha");
  }

  @Test
  public void test_ex_table() {
    testFile("examples/table.haha");
  }

  @Test
  public void test_ex_partition() {
    testFile("examples/partition.haha");
  }

}
