package net.afyre.haha.demo.ui.handler;


import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.nio.file.FileAlreadyExistsException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModelExtension;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleConstants;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.IConsoleView;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.services.ISourceProviderService;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.nodemodel.util.NodeModelUtils;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextEditor;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;

import net.afyre.haha.demo.coq.VCCoqPrinter;
import net.afyre.haha.demo.haha.Program;
import net.afyre.haha.demo.ui.HahaSourceProvider;
import net.afyre.haha.demo.ui.annotations.UnknownAssertAnnotation;
import net.afyre.haha.demo.ui.internal.HahaActivator;
import net.afyre.haha.demo.ui.prefs.CoqPreferencePage;
import net.afyre.haha.demo.util.CmpUtil;
import net.afyre.haha.demo.util.FileFinder;
import net.afyre.haha.demo.vcgen.HoareVCGen;
import net.afyre.haha.demo.vcgen.VCCollector;
import net.afyre.haha.demo.vcgen.VCContext;
import net.afyre.haha.demo.vcgen.VCPrinter;
import net.afyre.haha.demo.vcgen.VerificationCondition;

/**
 * ??
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * @author Aleksy Schubert <alx@mimuw.edu.pl>
 */
public class CoqVCGenHandler extends AbstractHandler implements IHandler {
private static final String CONSOLE_NAME = "VerificationConditions";
  private FileFinder fileFinder = null;
  
  /** {@inheritDoc} */
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
	if (fileFinder==null) {
		fileFinder = new FileFinder();
	}
    final ISourceProviderService sourceProviderService = 
        (ISourceProviderService) 
          HandlerUtil.getActiveWorkbenchWindow(event).getService(
            ISourceProviderService.class);
    final HahaSourceProvider sourceProvider = 
      (HahaSourceProvider)
        sourceProviderService.getSourceProvider(HahaSourceProvider.VAR_RUNNING);
    
    final Object editorObject =  
      HandlerUtil.getVariable(event, "activeEditor");
    if (! (editorObject instanceof XtextEditor)) {
      MessageBox box = new MessageBox(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
      box.setMessage("Oops: " + editorObject.getClass().getCanonicalName());
      box.open(); 
      return null;
    }
    final XtextEditor editor = (XtextEditor)editorObject;
    final IXtextDocument doc = editor.getDocument();
    final IResource editorResource = editor.getResource();
    final IAnnotationModel annotationModel = 
        editor.getDocumentProvider().getAnnotationModel(editor.getEditorInput());
    final Thread thread = new Thread() {
      @Override
      public void run() {
        doc.readOnly(new IUnitOfWork<Object, XtextResource>() {
          public Object exec(XtextResource resource) {
            sourceProvider.setRunning(true);
            try {
              Program prog = (Program) resource.getContents().get(0);
              runVCGen(editorResource, annotationModel, prog, HandlerUtil
                  .getActiveWorkbenchWindow(event).getActivePage());
            } finally {
              sourceProvider.setRunning(false);
            }
            return null;
          }
        });
      }
    };
    sourceProvider.setThread(thread);
    thread.start();
    return null;
  }

  /**
   * The method prints a particular message to the given writer
   * with handling some special cases.
   * 
   * @param w the writer to write to
   * @param msg the message to write
   */
  private void print(Writer w, String msg) {
    final String message = msg == null ? "Null" : msg;
    try {
      w.write("CoqVCGen> " + message);
      w.flush();
    } catch (IOException e) {
      // Ignore.
    }
  }
  
  private void runVCGen(IResource resource,
                        IAnnotationModel annotationModel,
                        Program program, 
                        IWorkbenchPage page) {
    final HoareVCGen vcgen = new HoareVCGen();
    
    final MessageConsole console = findConsole(CONSOLE_NAME);
    final MessageConsoleStream out = 
        console.newMessageStream();
    final Writer writer = new OutputStreamWriter(out);


    print(writer, "Line number: " + getLineNo(page) + "\n");
    
	final IPreferenceStore store = 
	        HahaActivator.getInstance().getPreferenceStore();
    final String coqFilePath = store.getDefaultString(CoqPreferencePage.PREF_COQ_FILE_PATH);
	final String mainFileName = getMainFileName(page);
    
    FileOutputStream coqout;
    coqout = getCoqFileOutput(mainFileName, console, writer, coqFilePath);
    if (coqout == null) {
    	showConsole(page, console);
    	return;
    }
    final Writer coqwriter = new OutputStreamWriter(coqout);
    
    final VCCollector collector = new VCCollector();
    final VCPrinter printer = new VCPrinter(writer);
    final VCCoqPrinter coqprinter = new VCCoqPrinter(coqwriter);
        
    print(writer, "====================================\n");
    print(writer, " Collecting verification conditions\n");
    print(writer, "====================================\n\n");
    try {
      vcgen.generateVerificationConditions(null, program, collector);
    } catch (final Exception e) {
      e.printStackTrace(new PrintWriter(writer)); 
      try { writer.flush(); } catch (final IOException e2) {}
    }
    print(writer, "Verification conditions computed.\n");
    print(writer, "Conditions: ");    
    
    for (final VerificationCondition cond : collector.getConditions()) {
      printer.processVerificationCondition(cond);
      coqprinter.processVerificationCondition(cond);
    }
    printer.end();
    clearMarkers(resource, annotationModel);
    coqprinter.end();
    try {
		coqwriter.close();
	    coqout.close();
	} catch (IOException e1) {
	      e1.printStackTrace(new PrintWriter(writer)); 
	      try { writer.flush(); } catch (final IOException e2) {}
	      showConsole(page, console);
	      return;
	}

    print(writer, "\n\n");

    print(writer, "================\n");
    print(writer, " Running solver\n");
    print(writer, "================\n\n");
 
    try { writer.flush(); } catch (final IOException e) { }
    
    showConsole(page, console);
    callCoqIDE(store, coqFilePath, mainFileName, page, out); 
    try { writer.flush(); } catch (final IOException e) { }
    if (callCoqc(store, coqFilePath, mainFileName, page, out)) {  
    	HashSet<String> axioms = callCoqcheck(store, coqFilePath, mainFileName, page, out);
    	try { writer.flush(); } catch (final IOException e) { }
	    print(writer, String.format("Total number of conditions: %d\n", 
                collector.getConditions().size()));
    	markErrors(axioms, coqprinter.getVCs(),resource,annotationModel,writer);
    }
  }

	private void markErrors(HashSet<String> axioms, Hashtable<String, VerificationCondition> hashtable,
			IResource resource, IAnnotationModel annotationModel, Writer writer) {
		int unknown = 0;
		int valid = 0;
		Enumeration<String> e = hashtable.keys();

		while (e.hasMoreElements()) {
			String string = e.nextElement();
			if (axioms.contains(string)) {
				createUnknownMarkers(resource, annotationModel, hashtable.get(string));
				unknown++;
			} else {
				valid++;
			}
		}
		print(writer, String.format("Valid: %d\n", valid));
		print(writer, String.format("Unknown: %d\n", unknown));
	}

private HashSet<String> callCoqcheck(IPreferenceStore store, String coqFilePath, 
		  String mainFileName, IWorkbenchPage page, MessageConsoleStream out) {
		// Build the command
		final String[] cmd = new String[5];
		cmd[0] = store.getString(CoqPreferencePage.PREF_COQ_CHECKER_PATH);
		cmd[1] = "-o";
		cmd[2] = "-admit";
		cmd[3] = "ZArith";
		
		cmd[4] = "vcs_" + mainFileName;
		File directory = new File(coqFilePath);
		Process pr = runCommand(directory, cmd);
		parseCheck(pr.getInputStream());
		return checkedAxioms;
  }
  
  private enum States {
	    INIT, AXIOMS
	}

	private void parseCheck(InputStream inputStream) {	
		BufferedInputStream bs = new BufferedInputStream(inputStream);
		Scanner scanner = new Scanner(bs);
		States state = States.INIT;
		while (scanner.hasNextLine()) {
			StringBuffer iline = new StringBuffer(scanner.nextLine().trim());
			state = interpretLine(state,iline);
		}
		scanner.close();
		try {
			inputStream.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private HashSet<String> checkedAxioms = null;

	private States interpretLine(States state, StringBuffer iline) {
		switch (state) {
		case INIT:
			if (iline.toString().compareTo("* Axioms:") == 0 ) {
				checkedAxioms = new HashSet<String>();
				return States.AXIOMS;
			}
			return state;
		case AXIOMS:
			String item = iline.substring(iline.indexOf(".")+1).toString();
			checkedAxioms.add(item);
			return state;
		}
		return States.INIT;
	}

	/**
	 * This method calls coqc program with a file that contains the
	 * verification conditions.
	 * 
	 * @param store        store object to retrieve the file name 
	 *                     of coqc
	 * @param coqFilePath  the path where the Coq file is located
	 * @param mainFileName the basename of the Coq file to call coqide for
	 * @param page         the workbench page to make possible error display
	 * @param out 
	 * @return true in case the compilation went correctly
	 */
  private boolean callCoqc(IPreferenceStore store, String coqFilePath, 
		  String mainFileName, IWorkbenchPage page, MessageConsoleStream out) {
		// Build the command
		final String[] cmd = new String[2];
		cmd[0] = store.getString(CoqPreferencePage.PREF_COQ_SOLVER_PATH);

		cmd[1] = coqFilePath + File.separator + "vcs_" + mainFileName + ".v";
		File directory = new File(coqFilePath);
		Process pr = runCommand(directory, cmd);
		try {
			int res = pr.waitFor();
			if (res!=0) {
				showError("Cannot compile VCs in file vcs_" + mainFileName + ".v", page, out);
				return false;
			} else {
				showError("VC compilation OK", page, out);
				return true;
			}
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
  }

	/**
	 * This method calls coqide program with a file that contains the
	 * verification conditions.
	 * 
	 * @param store        store object to retrieve the file name 
	 *                     of coqide
	 * @param coqFilePath  the path where the Coq file is located
	 * @param mainFileName the basename of the Coq file to call coqide for
	 * @param page         the workbench page to make possible error display
	 * @param out 
	 */
	private void callCoqIDE(IPreferenceStore store, String coqFilePath, 
			String mainFileName, IWorkbenchPage page, MessageConsoleStream out) {
		// Fetch the file with tactics.
		bringTacticsFile(coqFilePath);

		// Build the command
		final String[] cmd = new String[2];
		cmd[0] = store.getString(CoqPreferencePage.PREF_COQ_IDE_PATH);

		cmd[1] = coqFilePath + File.separator + "vcs_" + mainFileName + ".v";
		File directory = new File(coqFilePath);
		Process pr = runCommand(directory, cmd);
		try {
			pr.waitFor();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private Process runCommand(File directory, final String[] cmd) {
		ProcessBuilder processBuilder = new ProcessBuilder(cmd);
		processBuilder.directory(directory);
		try {
			return processBuilder.start();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	private void bringTacticsFile(String coqFilePath) {
		InputStream stream;
		try {
			stream = FileLocator.openStream(HahaActivator.getInstance().getBundle(), new Path("resources/tactics.v"),
					false);
			File dest = new File(coqFilePath + File.separator + "tactics.v");

			Files.copy(stream, dest.toPath());

		} catch (FileAlreadyExistsException e0) {
			//We silently ignore it.
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	private FileOutputStream getCoqFileOutput(String mainFileName, 
			final MessageConsole console, final Writer writer,
			final String coqFilePath) {
		FileOutputStream coqout = null;
		String fname = coqFilePath + File.separator + "vcs_" + mainFileName;
		renameIfExistsCoq(fname);
		try {
			coqout = new FileOutputStream(fname + ".v");
		} catch (FileNotFoundException e) {
			print(writer, "Coq file: " + fname  + ".v" + " cannot be created.\n");
			print(writer, "Check if the directory exists.\n");
			return null;
		}
		print(writer, "Coq file: " + fname  + ".v" + "\n");
		return coqout;
	}
 
	/**
	 * The procedure tries to rename the current file under fname.v
	 * name to the first available one of the name fname<num>.v.
	 * The method does not copy the current file in case the last
	 * file of the name fname<num>.v has the same contents as the current
	 * file under fname.v
	 * 
	 * @param fname the basename of a file to rename
	 */
  private void renameIfExistsCoq(String fname) {
	  System.out.println(fname);
	  String namev = fname+".v";
	  File f = fileFinder.findFileOnPath(namev);
	  int num = 0;
	  while (f.exists()) {
		  num++;
		  f = fileFinder.findFileOnPath(fname + num + ".v");
	  }
	  if (num>=1) {
		  InputStream source = null;
		  try {
			  source = new FileInputStream(namev);
		  } catch (FileNotFoundException e1) {
			  // We silently ignore the impossible exception.
		  }
		  if (num==1 || CmpUtil.differFiles(namev,fname + (num-1) + ".v")) {
			  File dest = new File(fname + num + ".v");
			  try {
				  Files.copy(source, dest.toPath());
			  } catch(Exception e) {
				  e.printStackTrace();
			  }
		  }
	  }
  }



private int line = 0;
  private IWorkbenchPage mpage = null;
  private int getLineNo(IWorkbenchPage page) {
	this.mpage = page;

    Display.getDefault().syncExec(new Runnable() {
    	
        public void run() {
        	IEditorPart editorPart = mpage.getActiveEditor();
            ITextSelection textSelection = (ITextSelection) editorPart
                    .getSite().getSelectionProvider().getSelection();
            line = textSelection.getStartLine();
        }
    });
    
    return line;
  }
  
  /**
   * Returns the main part of the file name edited in the given page,
   * omitting its extension.
   * 
   * @param page the page to look for the file name in
   * @return the resulting name
   */
  private String getMainFileName(IWorkbenchPage page) {
	  String fullname = 
			  	page.getActiveEditor().getEditorInput().getName();
	  int endpos = fullname.lastIndexOf('.');
	  return fullname.substring(0, endpos);
  }

  private void clearMarkers(final IResource resource, 
                            final IAnnotationModel annotationModel) {
    if (resource != null) { 
      try {
        resource.deleteMarkers("net.afyre.haha.demo.ui.hahamarker",
                                                   true, 
                                                   IResource.DEPTH_INFINITE);
      } catch (final CoreException e) {
        e.printStackTrace();
      }
    } else  {
      final List<Annotation> annotations = new ArrayList<Annotation>();
      final Iterator<Annotation> it = annotationModel.getAnnotationIterator();
      while (it.hasNext()) {
        final Annotation annotation = it.next();
        if (annotation.getType().startsWith("net.afyre.haha")) {
          annotations.add(annotation);
        }
      }
      if (annotationModel instanceof IAnnotationModelExtension) {
        ((IAnnotationModelExtension)annotationModel).replaceAnnotations(
            annotations.toArray(new Annotation[]{}), null);
      } else {
        for (final Annotation annotation : annotations) {
          annotationModel.removeAnnotation(annotation);
        }
      }
    }
  }
  
  private void createUnknownMarkers(final IResource resource,
                                    final IAnnotationModel annotationModel, 
                                    final VerificationCondition vc) {
    if (resource != null) {
      for (final EObject srcElement : vc.getGoal().getSource()) {
        INode node = NodeModelUtils.findActualNodeFor(srcElement);
        
        try {
          final IMarker marker = 
            resource.createMarker("net.afyre.haha.demo.ui.unprovableassertmarker");
          String msg = "Unable to prove the verification condition.\n";
          msg += getVCDescription(vc);
          marker.setAttribute(IMarker.MESSAGE, msg);
          marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
          marker.setAttribute(IMarker.LINE_NUMBER, node.getStartLine());
          marker.setAttribute(IMarker.CHAR_START, node.getOffset());
          marker.setAttribute(IMarker.CHAR_END, node.getTotalEndOffset());
        } catch (CoreException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    } else {
      for (final EObject srcElement : vc.getGoal().getSource()) {
        final INode node = NodeModelUtils.findActualNodeFor(srcElement);
        final Position position = 
            new Position(node.getOffset(), node.getLength());
        final Annotation annotation = new UnknownAssertAnnotation(vc);
        annotationModel.addAnnotation(annotation, position);
      }
    }
  }
  

  
  private String getVCDescription(final VerificationCondition vc) {
    final List<String> contexts = new ArrayList<String>();
    
    VCContext current = vc.getContext();
    while (current != null) {
      contexts.add(current.getComment() + "\n");
      current = current.getParent();
    }
    final StringBuilder result = new StringBuilder();
    for (int i = contexts.size() - 1; i >= 0; i--) {
      result.append(contexts.get(i));
    }
    return result.toString();
  }
  
  private void showConsole(final IWorkbenchPage page, 
                           final MessageConsole console) {
    final String id = IConsoleConstants.ID_CONSOLE_VIEW;
    final Display display = page.getWorkbenchWindow().getShell().getDisplay();
    
    display.syncExec(new Runnable() {
      @Override
      public void run() {
        IConsoleView view;
        try {  
          view = (IConsoleView) page.showView(id);
          view.display(console);
        } catch (final PartInitException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    });  
  }
  
  private MessageConsole findConsole(String name) {
    ConsolePlugin plugin = ConsolePlugin.getDefault();
    IConsoleManager conMan = plugin.getConsoleManager();
    IConsole[] existing = conMan.getConsoles();
    for (int i = 0; i < existing.length; i++)
       if (name.equals(existing[i].getName()))
          return (MessageConsole) existing[i];
    //no console found, so create a new one
    MessageConsole myConsole = new MessageConsole(name, null);
    conMan.addConsoles(new IConsole[]{myConsole});
    return myConsole;
 }
  
  private void showError(String message, IWorkbenchPage page, MessageConsoleStream out) {
	  out.print("Remark: ");
	  out.println(message);
  }
}
