package net.afyre.haha.demo.ui.highlighting;

import java.util.List;

import net.afyre.haha.demo.haha.Assertion;
import net.afyre.haha.demo.haha.HahaPackage;
import net.afyre.haha.demo.haha.Program;

import org.eclipse.xtext.EcoreUtil2;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.nodemodel.util.NodeModelUtils;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.syntaxcoloring.IHighlightedPositionAcceptor;
import org.eclipse.xtext.ui.editor.syntaxcoloring.ISemanticHighlightingCalculator;

/**
 * Semantic highlighter logic.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class SemanticHighlightingCalculator implements
    ISemanticHighlightingCalculator {

  /** {@inheritDoc} */
  @Override
  public void provideHighlightingFor(
      final XtextResource resource,
      final IHighlightedPositionAcceptor acceptor) {
    if (resource == null) {
      return;
    }
    final Program program = (Program)resource.getContents().get(0);
    final Iterable<Assertion> assertions = 
        EcoreUtil2.eAllOfType(program, Assertion.class);
    for (final Assertion assertion : assertions) {
      final INode node = NodeModelUtils.findActualNodeFor(assertion);
      acceptor.addPosition(node.getOffset(), node.getLength(),
                           HighlightingConfiguration.ASSERTION); 
      final List<INode> nameNodes = 
        NodeModelUtils.findNodesForFeature(
            assertion, HahaPackage.Literals.ASSERTION__NAME);
      if (nameNodes.size() > 0) {
        final INode nameNode = nameNodes.get(0);
        acceptor.addPosition(nameNode.getOffset(), nameNode.getLength(),
                             HighlightingConfiguration.FORMULA_NAME); 
        
      }
      // TODO: Names of invariants and pre-post-conditions
      // TODO: Use lexer to do this?
    }
    
    /*final Iterable<INode> allNodes = 
      resource.getParseResult().getRootNode().getAsTreeIterable();
    for (final INode node : allNodes) {
      if (node.getGrammarElement() instanceof Assertion) {
        acceptor.addPosition(node.getOffset(), node.getLength(),
                             HighlightingConfiguration.ASSERTION);        
      }
    }*/
  }

}
