package net.afyre.haha.demo.ui.prefs;

import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.xtext.ui.editor.preferences.fields.LabelFieldEditor;

import net.afyre.haha.demo.ui.internal.HahaActivator;

/**
 * Preference page with solver settings.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class SolverPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage  {
  /** Solver path preference name. */
  public static final String PREF_SOLVER_PATH = "SOLVER_PATH";
  public static final String PREF_SOLVER_VERSION = "SOLVER_VERSION";
  public static final String PREF_SOLVER_PATH_DEFAULT = "z3";
  public static final String PREF_SOLVER_PATH_DEFAULT_WIN = "z3.exe";
  public static final String PREF_SOLVER_VERSION_DEFAULT = "4.3.2";

  public SolverPreferencePage() {
    super("Z3 Preferences", FieldEditorPreferencePage.FLAT);
    final URL url = 
        FileLocator.find(HahaActivator.getInstance().getBundle(), 
                         new Path("icons/z3-16x16.gif"), null);
    final ImageDescriptor image = ImageDescriptor.createFromURL(url);
    setImageDescriptor(image);
  }
  
  /** {@inheritDoc} */
  @Override
  protected void createFieldEditors() {
    final Composite parent = getFieldEditorParent();
    final ExecutableFieldEditor f1 = 
    		new ExecutableFieldEditor(PREF_SOLVER_PATH, "Z3 executable:", parent);
    final StringFieldEditor f2 =
    		new StringFieldEditor(PREF_SOLVER_VERSION, "Z3 version:", parent);
    f1.setStringValue(PREF_SOLVER_PATH_DEFAULT);
    f2.setStringValue(PREF_SOLVER_VERSION_DEFAULT);
    f2.fillIntoGrid(parent, 3);
    addField(f1);
    addField(new LabelFieldEditor("", parent)); // this is a dirty trick to force acceptable look
    addField(f2);
  }
  
  /** {@inheritDoc} */
  @Override
  public void init(IWorkbench workbench) {
    setPreferenceStore(HahaActivator.getInstance().getPreferenceStore());
  }
  
}
