package net.afyre.haha.demo.util.tests;

import static net.afyre.haha.demo.util.tests.Util.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.afyre.haha.demo.util.AbstractHashBuilder;

import org.junit.Test;

/**
 * Tests for the {@link AbstractHashBuilder} class.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class AbstractHashBuilderTest {
  /** AppendInt appends an integer using BE byte ordering. */
  @Test
  public void testAppendInt() {
    final TestBuilder builder = new TestBuilder();
    builder.appendInt(0x12345678);
    assertIteratorReturns("Wrong bytes placed in builder", 
                          builder.getBytes(), 
                          (byte)0x78, (byte)0x56, 
                          (byte)0x34, (byte)0x12);
  }
  
  private static class TestBuilder extends AbstractHashBuilder {
    private List<Byte> bytes = new ArrayList<Byte>();
    
    /**
     * Get all bytes from this builder.
     * @return Bytes.
     */
    public Iterator<Byte> getBytes() {
      return bytes.iterator();
    }
    
    /** {@inheritDoc} */
    @Override
    public void appendByte(byte b) {
      bytes.add(b);
    }

    /** {@inheritDoc} */
    @Override
    public void reset() {
      bytes.clear();
    }
    
  }
}
