package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;
import static org.junit.Assume.*;

import net.afyre.haha.demo.util.CompositeFunction;
import net.afyre.haha.demo.util.Function;

import org.junit.Test;


/**
 * Tests for the {@link CompositeFunction} class.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class CompositeFunctionTest {
  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final Function<String, String> f = 
        CompositeFunction.getInstance(new Reverse(), new Upcase());
    assertNotNull("Null returned from getInstance", f);
  }
  
  /** NPE is throw if the first function is null. */
  @Test(expected=NullPointerException.class)
  public void testFirstNull() {
    CompositeFunction.getInstance(null, new Upcase());    
  }
  
  /** NPE is throw if the second function is null. */
  @Test(expected=NullPointerException.class)
  public void testSecondNull() {
    CompositeFunction.getInstance(new Reverse(), null);    
  }
  
  /** Both functions are applied. */
  @Test
  public void testApply() {
    final Function<String, String> f = 
        CompositeFunction.getInstance(new Reverse(), new Upcase());
    assumeNotNull("Null returned from getInstance", f);
    final String result = f.apply("HelloWorld");
    assertEquals("Wrong application result", 
                 "DLROWOLLEH", result);
  }
  
  private static class Reverse implements Function<String, String> {
    /** {@inheritDoc} */
    @Override
    public String apply(final String arg) {
      final StringBuilder builder = new StringBuilder(arg.length());
      builder.append(arg);
      builder.reverse();
      return builder.toString();
    }
  }
  
  private static class Upcase implements Function<String, String> {
    /** {@inheritDoc} */
    @Override
    public String apply(final String arg) {
      return arg.toUpperCase();
    }
  }
    
  
}
