package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;

import java.util.NoSuchElementException;

import net.afyre.haha.demo.util.EmptyIterator;

import org.junit.Test;

/**
 * Tests for the {@link EmptyIterator} class.
 * Test cases defined here are usually duplicated
 * in EmptyIterableTest.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * @see EmptyIterableTest
 */
public class EmptyIteratorTest {

  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final EmptyIterator<?> it = EmptyIterator.getInstance();
    assertNotNull("Null returned from getInstance()", it);
  }

  /** HasNext() returns false. */
  @Test
  public void testHasNextIsFalse() {
    final EmptyIterator<?> it = EmptyIterator.getInstance();
    assertFalse("HasNext() returned wrong result", it.hasNext());
  }
  
  /** Next raises an exception. */
  @Test(expected=NoSuchElementException.class)
  public void testNextException() {
    final EmptyIterator<?> it = EmptyIterator.getInstance();
    it.next();
  }
  
  /** Remove throws an exception. */
  @Test
  public void testRemoveException() {
    final EmptyIterator<?> it = EmptyIterator.getInstance();
    boolean ok = false;
    
    try {
      it.remove();
    } catch (final UnsupportedOperationException e) {
      ok = true;
    } catch (final IllegalStateException e) {
      ok = true;
    } catch (final Exception e) {
      fail("Wrong exception thrown");
    }
    assertTrue("No exception thrown", ok);
  }
}
