/**
 * 
 */
package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;
import static org.junit.Assume.*;

import net.afyre.haha.demo.util.HashUtil;

import org.junit.Test;

/**
 * Tests for the {@link HashUtil} class.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class HashUtilTest {
  
  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final HashUtil util = HashUtil.getInstance();
    assertNotNull("Null returned from getInstance", util);
  }
  
  /** GetHashBuilder does not return null. */
  @Test
  public void testHashBuilderNotNull() {
    final HashUtil util = HashUtil.getInstance();
    assumeNotNull("Null returned from getInstance", util);
    assertNotNull("Hash builder is null", util.getHashBuilder());
  }  
  
  /** Hash can be computed even for null. */
  @Test
  public void testHashNull() {
    final HashUtil util = HashUtil.getInstance();
    assumeNotNull("Null returned from getInstance", util);
    try {
      util.hash((Integer)null);
    } catch (final Exception e) {
      fail("Exception thrown during hash computation: " + e.toString());
    }
  }
  
  /** 
   * Hash value is identical for two different, 
   * structurally equal arrays. 
   */
  @Test
  public void testHash() {
    final HashUtil util = HashUtil.getInstance();
    assumeNotNull("Null returned from getInstance", util);
    final Object[] array1 = { 1, 2, 3, 4, 5 };
    final Object[] array2 = { 1, 2, 3, 4, 5 };
    assertEquals("???", util.hash(array1), util.hash(array2));
  }

  /** 
   * Hash value is identical for two different, 
   * structurally equal byte arrays. 
   */
  @Test
  public void testHashBytes() {
    final HashUtil util = HashUtil.getInstance();
    assumeNotNull("Null returned from getInstance", util);
    final byte[] array1 = { 1, 2, 3, 4, 5 };
    final byte[] array2 = { 1, 2, 3, 4, 5 };
    assertEquals("???", 
                 util.hashBytes(array1), 
                 util.hashBytes(array2));
  }
}
