/**
 * 
 */
package net.afyre.haha.demo.util;

/**
 * Represents composition of two functions.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <Arg> Domain of the first function.
 * @param <T> Codomain of the first function.
 * @param <Result> Domain of the second function.
 */
public final class CompositeFunction<Arg, T, Result> 
    implements Function<Arg, Result> {
  /** First function. */
  private final Function<? super Arg, T> first;
  
  /** Second function. */
  private final Function<? super T, ? extends Result> second;
  
  /**
   * Constructor.
   * @param first First function.
   * @param second Second function.
   * @throws NullPointerException If either of the arguments is null.
   */
  private CompositeFunction(
      final Function<? super Arg, T> first,
      final Function<? super T, ? extends Result> second) {
    if (first == null) {
      throw new NullPointerException("first");
    }
    if (second == null) {
      throw new NullPointerException("second");
    }
    this.first = first;
    this.second = second;
  }
  
  /**
   * Create an instance of this class.
   * @param <Arg> Domain of the first function.
   * @param <T> Codomain of the first function.
   * @param <Result> Domain of the second function.
   * @param first First function.
   * @param second Second function.
   * @return Composite function object.
   * @throws NullPointerException If either of the arguments is null.
   */
  public static<Arg, T, Result> Function<Arg, Result> getInstance(
      final Function<? super Arg, T> first,
      final Function<? super T, ? extends Result> second) {
    return new CompositeFunction<Arg, T, Result>(first, second);
  }
  
  /** {@inheritDoc} */
  public Result apply(final Arg arg) {
    return second.apply(first.apply(arg));
  }
}
