/**
 * 
 */
package net.afyre.haha.demo.util;

import java.util.Iterator;

/**
 * An iterable wrapper which applies a transformation
 * to each element returned.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 * @param <Src> Type of elements returned by the original iterable.
 * @param <T> Type of elements returned by this iterable.
 */
public class MappedIterable<Src, T> implements Iterable<T> {
  /** Source iterable. */
  private final Iterable<? extends Src> source;
  
  /** Mapping function. */
  private final Function<? super Src, ? extends T> function;
  
  /**
   * Constructor.
   * @param source Source iterable.
   * @param function Mapping function.
   * @throws NullPointerException If either of the arguments is null.
   */
  private MappedIterable(
      final Iterable<? extends Src> source,
      final Function<? super Src, ? extends T> function) {
    if (source == null) {
      throw new NullPointerException("source");
    }
    if (function == null) {
      throw new NullPointerException("function");
    }
    this.source = source;
    this.function = function;
  }
  
  /**
   * Create a mapped iterable.
   * @param <Src> Type of elements returned by the source iterable.
   * @param <T> Type of elements returned by this iterable.
   * @param source Source iterable.
   * @param function Mapping function.
   * @throws NullPointerException If either of the arguments is null.
   */
  public static<Src, T> Iterable<T> getInstance(
      final Iterable<? extends Src> source,
      final Function<? super Src, ? extends T> function) {
    return new MappedIterable<Src, T>(source, function);
  }
  
  /** {@inheritDoc} */
  @Override
  public Iterator<T> iterator() {
    return MappedIterator.getInstance(source.iterator(), function);
  }

}
