package net.afyre.haha.demo.util;

import java.util.Iterator;

/**
 * Combines two iterables into an iterable over pairs.. 
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 * @param <L> Elements of the first iterable.
 * @param <R> Elements of the second iterable.
 */
public final class ZipIterable<L, R> implements Iterable<Pair<L, R>> {
  /** First iterable. */
  private final Iterable<? extends L> left;
  
  /** Second iterable. */
  private final Iterable<? extends R> right;
  
  /**
   * Constructor.
   * @param left First iterable.
   * @param right Second iterable.
   * @throws NullPointerException If given a null argument.
   */
  private ZipIterable(final Iterable<? extends L> left, 
                      final Iterable<? extends R> right) {
    if (left == null) { throw new NullPointerException("left"); }
    if (right == null) { throw new NullPointerException("right"); }
    this.left = left;
    this.right = right;
  }

  /**
   * Combine two iterables into an iterable over pairs.
   * @param <L> Elements of the first iterable.
   * @param <R> Elements of the second iterable.
   * @param left First iterable.
   * @param right Second iterable.
   * @return Combined iterable.
   * @throws NullPointerException If given a null argument.
   */
  public static<L, R> Iterable<Pair<L, R>> getInstance(
      final Iterable<? extends L> left, 
      final Iterable<? extends R> right) {
    return new ZipIterable<L, R>(left, right);
  }
  
  /** {@inheritDoc} */
  @Override
  public Iterator<Pair<L, R>> iterator() {
    return ZipIterator.getInstance(left.iterator(), right.iterator());
  }

}