package net.afyre.haha.demo.util.symboltable;

import java.util.Hashtable;
import java.util.Map;

/**
 * Simple symbol table implementation with no optimizations.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <Symbol> Type of symbols (identifiers).
 * @param <SymbolInfo> Type of information associated with a symbol.
 */
public final class SimpleFrame<Symbol, SymbolInfo> 
  implements Frame<Symbol, SymbolInfo> {

  /** Parent frame (null for root frames). */
  private final SimpleFrame<Symbol, SymbolInfo> parent;
  
  /** Map of symbols declared in this frame. */
  private final Map<Symbol, SymbolInfo> bindings;
  
  /**
   * Create a new, empty root frame.
   * @param <Symbol> Type of symbols (identifiers).
   * @param <SymbolInfo> Type of information associated with a symbol.
   * @return New frame.
   */
  public static<Symbol, SymbolInfo> 
  SimpleFrame<Symbol, SymbolInfo> createRootFrame() {
    return new SimpleFrame<Symbol, SymbolInfo>(null);
  }
  
  private SimpleFrame(final SimpleFrame<Symbol, SymbolInfo> parent) {
    this.parent = parent;
    this.bindings = new Hashtable<Symbol, SymbolInfo>();
  }
  
  /** {@inheritDoc} */
  @Override
  public Frame<Symbol, SymbolInfo> createChildFrame() {
    return new SimpleFrame<Symbol, SymbolInfo>(this);
  }

  /** {@inheritDoc} */
  @Override
  public Frame<Symbol, SymbolInfo> getParent() {
    return parent;
  }

  /** {@inheritDoc} */
  @Override
  public void setSymbolInfo(final Symbol symbol, 
                            final SymbolInfo info) {
    if (symbol == null) {
      throw new NullPointerException("symbol");
    }
    if (info == null) {
      throw new NullPointerException("info");
    }
    
    bindings.put(symbol, info);
  }

  /** {@inheritDoc} */
  @Override
  public SymbolInfo find(final Symbol symbol) {
    if (symbol == null) {
      throw new NullPointerException("symbol");
    }
    SimpleFrame<Symbol, SymbolInfo> current = this;
    SymbolInfo result = null;
    do {
      result = current.bindings.get(symbol); 
      current = current.parent;
    } while (current != null && result == null);
    return result;
  }
}
