/*
 * generated by Xtext
 */
package net.afyre.haha.demo.scoping;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import net.afyre.haha.demo.haha.Argument;
import net.afyre.haha.demo.haha.Axiom;
import net.afyre.haha.demo.haha.Exists;
import net.afyre.haha.demo.haha.Forall;
import net.afyre.haha.demo.haha.Function;
import net.afyre.haha.demo.haha.Local;
import net.afyre.haha.demo.haha.Locals;
import net.afyre.haha.demo.haha.Loop;
import net.afyre.haha.demo.haha.Postcondition;
import net.afyre.haha.demo.haha.Precondition;
import net.afyre.haha.demo.haha.Predicate;
import net.afyre.haha.demo.haha.Program;
import net.afyre.haha.demo.haha.QuantifiedVar;
import net.afyre.haha.demo.haha.Quantifier;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.xtext.naming.QualifiedName;
import org.eclipse.xtext.resource.EObjectDescription;
import org.eclipse.xtext.resource.IEObjectDescription;
import org.eclipse.xtext.scoping.IScope;
import org.eclipse.xtext.scoping.impl.AbstractDeclarativeScopeProvider;
import org.eclipse.xtext.scoping.impl.SimpleScope;
import org.eclipse.xtext.scoping.impl.SingletonScope;

import com.google.common.collect.Iterables;

// TODO: Redesign, comment, replace name-based dispatch with annotations.

class CompositeScope implements IScope {
  private final IScope[] scopes;

  public CompositeScope(final IScope... scopes) {
    this.scopes = scopes;
  }

  public Iterable<IEObjectDescription> getAllElements() {
    Map<QualifiedName, IEObjectDescription> result =
        new TreeMap<QualifiedName, IEObjectDescription>();
    for (final IScope scope : scopes) {
      for (final IEObjectDescription elem : scope.getAllElements()) {
        if (! result.containsKey(elem.getName())) {
          result.put(elem.getName(), elem);
        }
      }
    }
    return result.values();
  }

  public Iterable<IEObjectDescription> getElements(final EObject object) {
    Map<QualifiedName, IEObjectDescription> result =
        new TreeMap<QualifiedName, IEObjectDescription>();
    for (final IScope scope : scopes) {
      for (final IEObjectDescription elem : scope.getElements(object)) {
        if (! result.containsKey(elem.getName())) {
          result.put(elem.getName(), elem);
        }
      }
    }
    return result.values();
  }

  public Iterable<IEObjectDescription> getElements(final QualifiedName name) {
    Iterable<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();
    for (final IScope scope : scopes) {
      result = Iterables.concat(result, scope.getElements(name));
    }
    return result;
  }

  public IEObjectDescription getSingleElement(final EObject object) {
    for (final IScope scope : scopes) {
      final IEObjectDescription result = scope.getSingleElement(object);
      if (result != null) {
        return result;
      }
    }
    return null;
  }

  public IEObjectDescription getSingleElement(final QualifiedName name) {
    for (final IScope scope : scopes) {
      final IEObjectDescription result = scope.getSingleElement(name);
      if (result != null) {
        return result;
      }
    }
    return null;
  }
}

/**
 * Scoping description for Haha.
 */
public class HahaScopeProvider extends AbstractDeclarativeScopeProvider {

  IScope scope_Argument(final Function function, final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    for (final Argument argument :
      function.getArguments().getArguments()) {
      result.add(EObjectDescription.create(argument.getName(), argument));
    }
    return new SimpleScope(IScope.NULLSCOPE, result);
  }

  IScope scope_Argument(final Predicate pred, final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    for (final Argument argument :
      pred.getArguments().getArguments()) {
      result.add(EObjectDescription.create(argument.getName(), argument));
    }
    return new SimpleScope(IScope.NULLSCOPE, result);
  }

  IScope scope_Assignable(final Function func,
                          final EReference ref) {
    final IEObjectDescription resultVar =
        EObjectDescription.create(func.getName(), func.resultVar());
    return
        new SingletonScope(resultVar,
                           new CompositeScope(scope_Local(func, ref),
                                              getScope(func.eContainer(), ref)));
  }

  IScope scope_Callable(final Function func, final EReference ref) {
    final Program prog = (Program)func.eContainer();
    return new CompositeScope(scope_Function(prog, ref),
                              scope_Predicate(prog, ref));
  }

  IScope scope_Callable(final Axiom axiom, final EReference ref) {
    final Program prog = (Program)axiom.eContainer();
    return new CompositeScope(scope_Function(prog, ref),
                              scope_Predicate(prog, ref));
  }
  
  IScope scope_Callable(final Predicate pred, final EReference ref) {
    final Program prog = (Program)pred.eContainer();
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    for (final Predicate decl : prog.getPredicates()) {
      if (decl == pred) { break; }
      result.add(EObjectDescription.create(decl.getName(), decl));
    }
    return new SimpleScope(scope_Function(prog, ref), result); 
  }
  
  IScope scope_QuantifiedVar(final Quantifier expr, final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    for (final QuantifiedVar variable : expr.getVars()) {
      result.add(EObjectDescription.create(variable.getName(), variable));
    }
    return new SimpleScope(IScope.NULLSCOPE, result);
  }

  IScope scope_Variable(final Loop loop, final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    if (loop.getCounter() != null) {
      result.add(EObjectDescription.create(
                   loop.getCounter().getVariable().getName(), 
                   loop.getCounter().getVariable()));
    }
    
    return new SimpleScope(getScope(loop.eContainer(), ref), result);
  }
  
  IScope scope_Function(final Program program,
                        final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    for (final Function func : program.getFunctions()) {
      result.add(EObjectDescription.create(func.getName(), func));
    }
    return new SimpleScope(IScope.NULLSCOPE, result);
  }

  IScope scope_Local(final Function function, final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();
    for (final Locals locals : function.getLocals()) {
      for (final Local local : locals.getVariables()) {
        result.add(EObjectDescription.create(local.getName(), local));
      }
    }
    return new SimpleScope(IScope.NULLSCOPE, result);
  }

  IScope scope_Predicate(final Program program,
                         final EReference ref) {
    final List<IEObjectDescription> result =
        new ArrayList<IEObjectDescription>();

    for (final Predicate pred : program.getPredicates()) {
      result.add(EObjectDescription.create(pred.getName(), pred));
    }
    return new SimpleScope(IScope.NULLSCOPE, result);
  }

  IScope scope_Variable(final Exists exists, final EReference ref) {
    return new SimpleScope(
                           getScope(exists.eContainer(), ref),
                           scope_QuantifiedVar(exists, ref).getAllElements());
  }

  IScope scope_Variable(final Forall forall, final EReference ref) {
    return new SimpleScope(
                           getScope(forall.eContainer(), ref),
                           scope_QuantifiedVar(forall, ref).getAllElements());
  }

  IScope scope_Variable(final Function func, final EReference ref) {
    final IEObjectDescription resultVar =
        EObjectDescription.create(func.getName(), func.resultVar());
    return new SingletonScope(
                              resultVar,
                              new CompositeScope(scope_Local(func, ref),
                                                 scope_Argument(func, ref),
                                                 getScope(func.eContainer(), ref)));
  }

  IScope scope_Variable(final Precondition pre, final EReference ref) {
    final Function func = (Function)pre.eContainer();
    return new CompositeScope(scope_Argument(func, ref),
                              getScope(func.eContainer(), ref));
  }
  
	IScope scope_Variable(final Postcondition post, final EReference ref) {
		final Function func = (Function) post.eContainer();
		final IEObjectDescription resultVar = EObjectDescription.create(
				func.getName(), func.resultVar());
		return new SingletonScope(resultVar, new CompositeScope(scope_Argument(
				func, ref), getScope(func.eContainer(), ref)));
	}
  
  //  IScope scope_Function(final Function func,
  //                        final EReference ref) {
  //    if (ref.getEContainingClass().getClassifierID() == HahaPackage.LVALUE) {
  //      final IEObjectDescription result =
  //          EObjectDescription.create(func.getName(), func);
  //      return new SingletonScope(result, IScope.NULLSCOPE);
  //    } else {
  //      return scope_Function((Program)func.eContainer(), ref);
  //    }
  //
  //  }

  IScope scope_Variable(final Predicate pred, final EReference ref) {
    return new CompositeScope(scope_Argument(pred, ref),
                              getScope(pred.eContainer(), ref));
  }


}
