package net.afyre.haha.demo.types;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * Error information from typechecker.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class TypeError {
  /** Error message. */
  private final String message;
  
  /** Source location. */
  private final EObject source;
  
  /** Feature in the source element. */
  private final EStructuralFeature feature;
  
  /** Index in the feature. */
  private final int index;

  /**
   * Constructor.
   * @param message Error message.
   * @param source Source object.
   * @param feature Feature of the source object.
   * @param index Index inside the feature.
   */
  public TypeError(final String message, final EObject source, 
                   final EStructuralFeature feature, final int index) {
    this.message = message;
    this.source = source;
    this.feature = feature;
    this.index = index;
  }

  /**
   * Get the error message.
   * @return Message text.
   */
  public String getMessage() {
    return message;
  }

  /**
   * Get the source object.
   * @return Model element.
   */
  public EObject getSource() {
    return source;
  }

  /**
   * Get the feature name.
   * @return Feature.
   */
  public EStructuralFeature getFeature() {
    return feature;
  }

  /**
   * Get index within the feature.
   * @return Index.
   */
  public int getIndex() {
    return index;
  }
}
