package net.afyre.haha.demo.vcgen;

import java.util.List;

import net.afyre.haha.demo.solver.expr.formulae.Formula;

import org.eclipse.emf.ecore.EObject;

/**
 * A formula with attached name, comment and source elements.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class DocumentedFormula {
  /** Formula content. */
  private final Formula formula;
  
  /** Short name. */
  private final String name;
  
  /** Source elements associated with this formula. */
  private final List<EObject> source;
  
  /** Descriptive comment. */
  private final String comment;
  
  /**
   * Create a documented formula.
   * @param formula Formula content.
   * @param name Short name.
   * @param source Associated source elements.
   * @param comment Descriptive comment.
   */
  public DocumentedFormula(final Formula formula,
                           final String name, 
                           final List<EObject> source, 
                           final String comment) {
    this.formula = formula;
    this.name = name;
    this.source = source;
    this.comment = comment;
  }

  /**
   * Get formula content.
   * @return Formula.
   */
  public Formula getFormula() {
    return formula;
  }
  
  /**
   * Get the short name of this formula.
   * @return Context name.
   */
  public String getName() {
    return name;
  }

  /**
   * Get list of source elements associated with this formula.
   * Returned list is not modifiable.
   * @return List of AST elements.
   */
  public List<EObject> getSource() {
    return source;
  }

  /**
   * Get the comment associated with this formula.
   * @return Comment text.
   */
  public String getComment() {
    return comment;
  }
}
