package net.afyre.haha.demo.vcgen;

import net.afyre.haha.demo.haha.Expression;
import net.afyre.haha.demo.haha.Type;
import net.afyre.haha.demo.solver.expr.formulae.Formula;
import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Utilities for handling expressions.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class ExpressionUtil {
  /** Singleton instance. */
  private static ExpressionUtil instance;
  
  private final ExpressionSwitch exprSwitch = 
      new ExpressionSwitch();
  
  /**
   * Create an instance of this class.
   * @return Instance.
   */
  public static synchronized ExpressionUtil getInstance() {
    if (instance == null) {
      instance = new ExpressionUtil();
    }
    return instance;
  }
  
  /**
   * Translate an expression to a SMT formula.
   * @param expr AST expression.
   * @return SMT formula.
   */
  public Formula translateFormula(final Expression expr) {
    return (Formula)exprSwitch.doSwitch(expr);
  }
  
  /**
   * Translate an expression to a SMT term.
   * @param expr AST expression.
   * @return SMT term.
   */
  public Term translateTerm(final Expression expr) {
    return (Term)exprSwitch.doSwitch(expr);
  }
    
  /**
   * Translate an AST type to a SMT type.
   * @param expr AST type.
   * @return SMT type.
   */
  public ValueType translateType(final Type expr) {
    return (ValueType)exprSwitch.doSwitch(expr);
  }
  
  /**
   * Translate any kind of AST expression to a SMT expression.
   * @param expr AST expression.
   * @return SMT expression.
   */
  public net.afyre.haha.demo.solver.expr.Expression
  translateExpression(final Expression expr) {
    return (net.afyre.haha.demo.solver.expr.Expression)
        exprSwitch.doSwitch(expr);    
  }
}
