package net.afyre.haha.demo.vcgen;

import java.io.IOException;
import java.io.Writer;

import net.afyre.haha.demo.solver.context.Declaration;
import net.afyre.haha.demo.solver.context.DeclarationPrinter;
import net.afyre.haha.demo.solver.expr.util.ExprPrinter;

// TODO: Better way of processing answers.

/**
 * Verification condition processor which prints
 * all conditions and contexts.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public final class VCPrinter extends VCProcessor {
  /** Output stream. */
  private final Writer output;

  /** Current indentation level. */
  private int indent;

  /** Visitor used to print formulae. */
  private final ExprPrinter exprPrinter;
  
  /** Visitor used to print declarations. */
  private final DeclarationPrinter declPrinter;
  
  /**
   * Constructor.
   * @param writer Output stream.
   */
  public VCPrinter(final Writer output) {
    this.output = output;
    this.indent = 0;
    this.exprPrinter = new ExprPrinter(output);
    this.declPrinter = new DeclarationPrinter(output);
  }
  
  // TODO: Move this to utils
  /**
   * Print an object (using toString()) to the output stream, 
   * converting any IOException to RuntimeExceptions.
   * 
   * @param obj Object to be printed (can be null).
   */
  private void print(final Object obj) {
    final String message;
    
    if (obj == null) {
      message = "null";
    } else {
      message = obj.toString();
    }
    try {
      output.write(message);
    } catch (final IOException e) {
      throw new RuntimeException(e);
    }
  }

  /**
   * Print indentation.
   */
  private void printIndent() {
    for (int i = 0; i < indent; i++) {
      print("  ");
    }
  }
  
  /** {@inheritDoc} */
  @Override
  protected void processCondition(
      final VerificationCondition vc) {
    for (final DocumentedFormula pre : vc.getAssumptions()) {
      printIndent();
      print(" ");
      print(pre.getName());
      print(" : ");
      pre.getFormula().accept(exprPrinter);
      print("\n");
    }
    printIndent();
    print("---------------------------------------------\n");
    printIndent();
    print(" ");
    vc.getGoal().getFormula().accept(exprPrinter);
    print("\n\n");
  }

  /** {@inheritDoc} */
  @Override
  protected void pushContext(final VCContext context) {
    printIndent();
    print("-- " + context.getComment() + "\n");
    printIndent();
    print(context.getName() + " {\n");
    indent++;
    for (final Declaration decl : context.getDeclarations()) {
      printIndent();
      decl.accept(declPrinter);
      print("\n");
    }
  }

  /** {@inheritDoc} */
  @Override
  protected void popContext(final VCContext context) {
    indent--;
    printIndent();
    print("}\n");
  }
}
